@@ -1,4 +1,4 @@
-/* stringprep.h		Header file for stringprep functions.         -*- c -*-
+/* stringprep.h --- Header file for stringprep functions.             -*- c -*-
  * Copyright (C) 2002, 2003, 2004  Simon Josefsson
  *
  * This file is part of GNU Libidn.
@@ -31,7 +31,7 @@ extern "C"
 #include <unistd.h>		/* ssize_t */
 #include <idn-int.h>		/* uint32_t */
 
-#define STRINGPREP_VERSION "0.5.5"
+#define STRINGPREP_VERSION "0.5.6"
 
 /* Error codes. */
   typedef enum
@@ -181,6 +181,8 @@ extern "C"
 				 const char *profile,
 				 Stringprep_profile_flags flags);
 
+  extern const char *stringprep_strerror (Stringprep_rc rc);
+
   extern const char *stringprep_check_version (const char *req_version);
 
 /* Utility */
